declare type ArgumentsType<T> = T extends (...args: infer A) => any ? A : never;
declare type ReturnType<T> = T extends (...args: any) => infer R ? R : never;
interface BirpcOptions<Functions> {
    /**
     * Local functions implementation.
     */
    functions: Functions;
    /**
     * Function to post raw message
     */
    post: (data: any) => void;
    /**
     * Listener to receive raw message
     */
    on: (fn: (data: any) => void) => void;
    /**
     * Custom function to serialize data
     *
     * by default it passes the data as-is
     */
    serialize?: (data: any) => any;
    /**
     * Custom function to deserialize data
     *
     * by default it passes the data as-is
     */
    deserialize?: (data: any) => any;
}
declare type BirpcFn<T> = {
    /**
     * Call the remote function and wait for the result.
     */
    (...args: ArgumentsType<T>): Promise<Awaited<ReturnType<T>>>;
    /**
     * Send event without asking for response
     */
    noReply(...args: ArgumentsType<T>): void;
};
declare type BirpcReturn<RemoteFunctions> = {
    [K in keyof RemoteFunctions]: BirpcFn<RemoteFunctions[K]>;
};
declare function createBirpc<LocalFunctions = {}, RemoteFunctions = {}>({ functions, post, on, serialize, deserialize, }: BirpcOptions<LocalFunctions>): BirpcReturn<RemoteFunctions>;

export { ArgumentsType, BirpcFn, BirpcOptions, BirpcReturn, ReturnType, createBirpc };
